@@ -47,7 +47,7 @@ class IbmDbTestCase(unittest.TestCase):
                 cols = ibm_db.fetch_tuple( stmt_select )
             
             #populate the tabmany table 
-            params = ((50, 'Hanes'), (55, ), (55.5, 'invalid row'), (60, 'Quigley') )
+            params = ((50, 'Hanes'), (55, ), (55.5, 'invalid row'), (60, 'Quigley'), (70, None) )
             try:
                 ibm_db.execute_many(stmt_insert, params)
             except Exception, inst:
@@ -70,7 +70,7 @@ class IbmDbTestCase(unittest.TestCase):
 #40, OBrien
 #Error 1: Value parameter tuple: 2 has less no of param 
 #Error 2: Value parameters array 3 is not homogeneous with privious parameters array 
-#2
+#3
 #__ZOS_EXPECTED__
 #4
 #10, Sanders
@@ -79,4 +79,4 @@ class IbmDbTestCase(unittest.TestCase):
 #40, OBrien
 #Error 1: Value parameter tuple: 2 has less no of param 
 #Error 2: Value parameters array 3 is not homogeneous with privious parameters array 
-#2
+#3
